
// A cigarette with some function media smoke.
// Made and copyright by Anders Haglund 1999.
//------------------------------------------------
// Updated to POV-Ray v3.x code
// -added functions.inc
// -changed megapov noise3d to POV-Ray v3 f_noise3d
// -reduced the size of the non-burnt cigarette
// -moved "noise_generator" from global settings to pigment bozo
// -changed the smoke from a cylinder to a cone
// -added max_gradient to isosurface
// -changed bounded_by to contained_by in isosurface
// -changed smoke color from white to blue

#version 3.6;

global_settings {
	assumed_gamma 2.2
 	//noise_generator 1
}

#include "functions.inc"

camera {
	location <7,4,-9>
	look_at  <0,3,0>
}

light_source { <20,30,-25> rgb 1 }  //original
//light_source { <20,30,-25> rgb 1 area_light <20,30,-25>, <20,30,-25>, 10, 10 adaptive 2 jitter circular orient spotlight radius 15 falloff 60 tightness 3 point_at <0,3,0> }

#declare burned = 0.2; // How long it's burned (0-1). Haven't realy tried this so it may not work...
#declare noisegain = 0.5; // Noise gain in y axis
#declare noisestart = 0.5; // Starting noise

// The smoke function. Could probably be done better but it works....
// Original megapov smoke
/*
#declare smoke =
function {
	sqrt( sqr(x+noisegain*(y+noisestart)*(noise3d(x,y,z)-0.5)) + sqr(z+noisegain*(y+noisestart)*(noise3d(z,y,z)-0.5)) )
}

#declare smoke =
function {
	sqrt(
		pow( (x+noisegain*(y+noisestart)*(f_noise3d(x,y,z)-0.5)),2)
		+
		pow( (z+noisegain*(y+noisestart)*(f_noise3d(z,y,z)-0.5)),2)
	)
}

#declare smoke =
function {
	sqrt(
		(x+noisegain*(y+noisestart)*(f_noise3d(x,y,z)-0.5))*(x+noisegain*(y+noisestart)*(f_noise3d(x,y,z)-0.5))
		+
		(z+noisegain*(y+noisestart)*(f_noise3d(z,y,z)-0.5))*(z+noisegain*(y+noisestart)*(f_noise3d(z,y,z)-0.5))
	)
}
*/
#declare smoke =
function {
	sqrt( (x+noisegain*(y+noisestart)*(f_noise3d(x,y,z)-0.5)) + (z+noisegain*(y+noisestart)*(f_noise3d(z,y,z)-0.5)) )
}

// The texture for the glow
#declare g1 = 
texture { 
	pigment { 
		granite
		color_map {
			[ 0 rgb <1,0,0> ]
			[ 1 rgb <1,0.5,0> ]
		}
	}
	finish {
		ambient 1
	}
}
#declare g2 = 
texture {
	pigment {
		crackle
		turbulence 0.5
		color_map {
			[ 0.05 rgbt <0,0,0,0.5> ]
			[ 0.15 rgbt <0.2,0.2,0.2,0> ]
		}
		scale 0.1
	}
}
#declare glow =
texture { g1 }
texture { g2 }

 // Cigarette
union {
	union {
		//cylinder { <0,0,0>, <5.9,0,0>, 0.4 //original
		cylinder { <1,0,0>, <5.9,0,0>, 0.4
			pigment {
				gradient <1,0,0>
				turbulence <0.06,1,1>
				color_map {
					[ burned/2      rgbt <0,0,0,1> ]
					//[ burned/2+0.01 rgb <0.4,0.2,0> ] //original
					[ burned/2+0.01 rgb <0.8,0.35,0> ]
					[ burned/2+0.02 rgb 0.95 ]
				}
				scale <11.6,1,1>
			}
			finish {
				diffuse 0.9
				brilliance 0.6
			}
			hollow
		}
		
		difference {
			cylinder { <5.9,0,0>, <8.3,0,0>, 0.4
				pigment {
					bozo
					noise_generator 1  //moved from global settings
					color_map {
						[ 0.90 rgb <0.7,0.5,0.2> ]
						[ 0.91 rgb <0.9,0.7,0.3> ]
					}
					scale <0.15,0.1,0.1>
				}
				finish {
					diffuse 0.9
					brilliance 0.6
				}
			}
			box { <8.2,0.5,0.5>, <8.4,-0.5,-0.5>
				pigment { rgb 1 }
				finish {
					brilliance 0.6
					diffuse 1
				}
				normal {
					granite 0.03
					scale 0.1
				}
			}
		}
		// The glow
		isosurface {
			//function { sqrt(x*x+y*y+z*z)+0.1*noise3d(x*10,y*10,z*10) } //original
			function { sqrt(x*x+y*y+z*z)+0.1*f_noise3d(x*10,y*10,z*10) }
			max_gradient 1.6 //!
			threshold 0.4
			//bounded_by {
			contained_by {
				sphere { <0,0,0>, 0.4 }
			}
			translate <(5.8*burned-0.35),0,0>
			texture { glow }
			scale <2,1,1>
		}
		translate <-4.2,0,0>
	}
	// The container for the smoke media
	//cylinder { <0,0,0>, <0,30,0>, 2 //original
	cylinder { <0,0.4,0>, <0,30,0>, 2
		pigment { rgbt 1 }
		hollow
		interior {
			media {
				samples 2, 20
				intervals 30
				scattering { 2 color rgb 0.8 }
				density {
					function { min(smoke(x,y,z),1) }
					color_map {
						//[ 0.0 rgb 1 ] //original
						[ 0.0 rgb <0.7,0.7,1>*1.4 ] //blue fug
						[ 0.5 rgb 0 ]
					}
				}
			}
		}
		scale <1,2,1>
		translate <(5.8*burned-4.2),0,0>
	}
}

plane {
	<0,1,0>, -4
	pigment { rgb 0.5 }
	//pigment { checker rgb 0.25 rgb 0.5 scale 5 }
	finish { ambient 0 }
}

#declare Glass=1;
#declare Bowl = lathe{
	cubic_spline
	20,
	<0.000000,0.082805>,<0.000000,0.109516>,<0.080134,0.109516>,<0.267112,0.109516>,<0.534224,0.162938>,
	<0.721202,0.323205>,<0.801336,0.536895>,<0.828047,0.697162>,<0.881469,0.777295>,<0.988314,0.777295>,
	<0.988314,0.750584>,<0.908180,0.750584>,<0.854758,0.697162>,<0.828047,0.536895>,<0.747913,0.296494>,
	<0.534224,0.109516>,<0.267112,0.056093>,<0.454090,0.002671>,<0.000000,0.002671>,<0.000000,-0.024040>
	#if (Glass)
		material {
			texture {
				pigment {rgbt 1}
				finish {
					ambient 0.0
					diffuse 0.05
					specular 0.6
					roughness 0.0005
					reflection {
						0.2,0.9
						fresnel on
					}
					conserve_energy
				}
			}
			interior {
				ior 1.5
				fade_power 1001
				fade_distance 0.9
				fade_color <0.5,0.8,0.6>
			}
		}
		photons {
			target 1.0
			refraction on
			reflection on
		}
	#else
		pigment { rgb 0.8 }
	#end
	scale <1,0.75,1>
}

object {
    Bowl
    scale 4
    translate <-3,-2.8,0>
}
